@@ -20,6 +20,7 @@ Circuit generation for measuring gate errors
 
 import numpy as np
 import qiskit
+from qiskit.circuit.library import U1Gate, U2Gate
 
 
 def ampcal_1Q_circuits(max_reps, qubits):
@@ -54,10 +55,10 @@ def ampcal_1Q_circuits(max_reps, qubits):
         circ = qiskit.QuantumCircuit(qr, cr)
         circ.name = 'ampcal1Qcircuit_' + str(circ_index) + '_0'
         for qind, qubit in enumerate(qubits):
-            circ.u2(0.0, 0.0, qr[qubit])
+            circ.append(U2Gate(0.0, 0.0), [qr[qubit]])
             for _ in range(circ_length):
                 circ.barrier(qr[qubit])
-                circ.u2(0.0, 0.0, qr[qubit])
+                circ.append(U2Gate(0.0, 0.0), [qr[qubit]])
 
         for qind, qubit in enumerate(qubits):
             circ.measure(qr[qubit], cr[qind])
@@ -97,22 +98,22 @@ def anglecal_1Q_circuits(max_reps, qubits, angleerr=0.0):
         circ = qiskit.QuantumCircuit(qr, cr)
         circ.name = 'anglecal1Qcircuit_' + str(circ_index) + '_0'
         for qind, qubit in enumerate(qubits):
-            circ.u2(0.0, 0.0, qr[qubit])  # Y90p
+            circ.append(U2Gate(0.0, 0.0), [qr[qubit]])  # Y90p
             for _ in range(circ_length):
                 if angleerr != 0:
-                    circ.u1(-2*angleerr, qr[qubit])
+                    circ.append(U1Gate(-2*angleerr), [qr[qubit]])
                 for _ in range(2):
                     circ.barrier(qr[qubit])
-                    circ.u2(-np.pi/2, np.pi/2, qr[qubit])  # Xp
+                    circ.append(U2Gate(-np.pi/2, np.pi/2), [qr[qubit]])  # Xp
                 if angleerr != 0:
-                    circ.u1(2*angleerr, qr[qubit])
+                    circ.append(U1Gate(2*angleerr), [qr[qubit]])
                 for _ in range(2):
                     circ.barrier(qr[qubit])
-                    circ.u2(0.0, 0.0, qr[qubit])  # Yp
+                    circ.append(U2Gate(0.0, 0.0), [qr[qubit]])  # Yp
 
             if angleerr != 0:
-                circ.u1(-angleerr, qr[qubit])
-            circ.u2(-np.pi/2, np.pi/2, qr[qubit])  # X90p
+                circ.append(U1Gate(-angleerr), [qr[qubit]])
+            circ.append(U2Gate(-np.pi/2, np.pi/2), [qr[qubit]])  # X90p
         for qind, qubit in enumerate(qubits):
             circ.measure(qr[qubit], cr[qind])
         circuits.append(circ)
@@ -159,7 +160,7 @@ def ampcal_cx_circuits(max_reps, qubits, control_qubits):
         circ.name = 'ampcalcxcircuit_' + str(circ_index) + '_0'
         for qind, qubit in enumerate(qubits):
             circ.x(qr[control_qubits[qind]])
-            circ.u2(-np.pi/2, np.pi/2, qr[qubit])  # X90p
+            circ.append(U2Gate(-np.pi/2, np.pi/2), [qr[qubit]])  # X90p
             for _ in range(circ_length):
                 circ.barrier([qr[control_qubits[qind]], qr[qubit]])
                 circ.cx(qr[control_qubits[qind]], qr[qubit])
@@ -214,17 +215,17 @@ def anglecal_cx_circuits(max_reps, qubits, control_qubits, angleerr=0.0):
         circ.name = 'anglecalcxcircuit_' + str(circ_index) + '_0'
         for qind, qubit in enumerate(qubits):
             circ.x(qr[control_qubits[qind]])
-            circ.u2(0.0, 0.0, qr[qubit])  # Y90p (target)
+            circ.append(U2Gate(0.0, 0.0), [qr[qubit]])  # Y90p (target)
             for _ in range(circ_length):
                 if angleerr != 0:
-                    circ.u1(-angleerr, qr[qubit])
+                    circ.append(U1Gate(-angleerr), [qr[qubit]])
                 circ.barrier([qr[control_qubits[qind]], qr[qubit]])
                 circ.cx(qr[control_qubits[qind]], qr[qubit])
                 if angleerr != 0:
-                    circ.u1(angleerr, qr[qubit])
+                    circ.append(U1Gate(angleerr), [qr[qubit]])
                 circ.y(qr[qubit])  # Yp (target)
 
-            circ.u2(-np.pi/2., np.pi/2., qr[qubit])  # X90p (target)
+            circ.append(U2Gate(-np.pi/2., np.pi/2.), [qr[qubit]])  # X90p (target)
         for qind, qubit in enumerate(qubits):
             circ.measure(qr[qubit], cr[qind])
         circuits.append(circ)
