@@ -10,6 +10,8 @@
 # copyright notice, and modified files need to carry a notice indicating
 # that they have been altered from the originals.
 
+# pylint: disable=invalid-name
+
 
 """
 Class for accreditation protocol
@@ -38,7 +40,7 @@ class AccreditationFitter:
 
     bound = 1
     confidence = 1
-    n_acc = 0
+    N_acc = 0
     num_runs = 0
     flag = 'accepted'
     outputs = []
@@ -90,7 +92,7 @@ class AccreditationFitter:
             else:
                 output_target = count
         if self.flag == 'accepted':
-            self.n_acc += 1
+            self.N_acc += 1
             self.outputs.append(output_target)
 
     def bound_variation_distance(self, theta):
@@ -101,12 +103,12 @@ class AccreditationFitter:
         Args:
             theta (float): number between 0 and 1
         """
-        if self.n_acc == 0:
+        if self.N_acc == 0:
             QiskitError("ERROR: Variation distance requires"
                         "at least one accepted run")
-        if self.n_acc/self.num_runs > theta:
+        if self.N_acc/self.num_runs > theta:
             self.bound = self.g_num*1.7/(self.num_traps+1)
-            self.bound = self.bound/(self.n_acc/self.num_runs-theta)
+            self.bound = self.bound/(self.N_acc/self.num_runs-theta)
             self.bound = self.bound+1-self.g_num
             self.confidence = 1-2*np.exp(-2*theta*self.num_runs*self.num_runs)
         if self.bound > 1:
