@@ -48,6 +48,7 @@ from qiskit.quantum_info.operators.operator import Operator
 from qiskit.quantum_info.operators.pauli import Pauli
 from qiskit.quantum_info.operators.scalar_op import ScalarOp
 from qiskit.circuit import QuantumCircuit, Instruction
+from qiskit.circuit.library import U1Gate
 
 
 class SpecialPolynomial():
@@ -400,8 +401,7 @@ class CNOTDihedral(BaseOperator):
             self.shift = elem.shift
 
         # Initialize BaseOperator
-        dims = self._num_qubits * (2,)
-        super().__init__(dims, dims)
+        super().__init__(num_qubits=self._num_qubits)
 
         # Validate the CNOTDihedral element
         if validate and not self.is_cnotdihedral():
@@ -590,7 +590,7 @@ class CNOTDihedral(BaseOperator):
             circuit = circuit.to_instruction()
 
         # Initialize an identity CNOTDihedral object
-        elem = CNOTDihedral(self.num_qubits)
+        elem = CNOTDihedral(self._num_qubits)
         append_circuit(elem, circuit)
         return elem
 
@@ -654,7 +654,7 @@ class CNOTDihedral(BaseOperator):
         other.poly.weight_0 = 0  # set global phase
         return other
 
-    def _tensor_product(self, other, reverse=False):
+    def _tensor(self, other, reverse=False):
         """Returns the tensor product operator.
 
          Args:
@@ -718,7 +718,7 @@ class CNOTDihedral(BaseOperator):
              CNOTDihedral: the tensor product operator: self tensor other.
          """
 
-        return self._tensor_product(other, reverse=True)
+        return self._tensor(other, reverse=True)
 
     def expand(self, other):
         """Return the tensor product operator: other tensor self.
@@ -729,7 +729,7 @@ class CNOTDihedral(BaseOperator):
              CNOTDihedral: the tensor product operator: other tensor other.
          """
 
-        return self._tensor_product(other, reverse=False)
+        return self._tensor(other, reverse=False)
 
     def adjoint(self):
         """Return the conjugate transpose of the CNOTDihedral element"""
@@ -743,8 +743,10 @@ class CNOTDihedral(BaseOperator):
         circ = self.to_instruction()
         new_circ = QuantumCircuit(self.num_qubits)
         qargs = list(range(self.num_qubits))
+        bit_indices = {bit: index
+                       for index, bit in enumerate(circ.definition.qubits)}
         for instr, qregs, _ in circ.definition:
-            new_qubits = [qargs[tup.index] for tup in qregs]
+            new_qubits = [qargs[bit_indices[tup]] for tup in qregs]
             if instr.name == 'u1':
                 params = 2 * np.pi - instr.params[0]
                 instr.params[0] = params
@@ -878,9 +880,13 @@ def append_circuit(elem, circuit, qargs=None):
         raise QiskitError('{} instruction definition is {}; expected QuantumCircuit'.format(
             gate.name, type(gate.definition)))
 
+    flat_instr = gate.definition
+    bit_indices = {bit: index
+                   for index, bit in enumerate(flat_instr.qubits)}
+
     for instr, qregs, _ in gate.definition:
         # Get the integer position of the flat register
-        new_qubits = [qargs[tup.index] for tup in qregs]
+        new_qubits = [qargs[bit_indices[tup]] for tup in qregs]
 
         if (instr.name == 'x' or gate.name == 'x'):
             if len(new_qubits) != 1:
@@ -984,7 +990,7 @@ def decompose_cnotdihedral_2_qubits(elem):
         tpow0 = elem.poly.weight_1[0]
         xpow0 = elem.shift[0]
         if tpow0 > 0:
-            circuit.u1(tpow0 * np.pi / 4, 0)
+            circuit.append(U1Gate(tpow0 * np.pi / 4), [0])
         if xpow0 == 1:
             circuit.x(0)
         if (tpow0 == 0 and xpow0 == 0):
@@ -1007,11 +1013,11 @@ def decompose_cnotdihedral_2_qubits(elem):
         if weight_2 == [0]:
             [tpow0, tpow1] = weight_1
             if tpow0 > 0:
-                circuit.u1(tpow0 * np.pi / 4, 0)
+                circuit.append(U1Gate(tpow0 * np.pi / 4), [0])
             if xpow0 == 1:
                 circuit.x(0)
             if tpow1 > 0:
-                circuit.u1(tpow1 * np.pi / 4, 1)
+                circuit.append(U1Gate(tpow1 * np.pi / 4), [1])
             if xpow1 == 1:
                 circuit.x(1)
             if (tpow0 == 0 and tpow1 == 0 and xpow0 == 0 and xpow1 == 0):
@@ -1024,18 +1030,18 @@ def decompose_cnotdihedral_2_qubits(elem):
             tpow0 = (weight_1[0] - 2 * xpow1 - 4 * xpow0 * xpow1) % 8
             tpow1 = (weight_1[1] - 2 * xpow0 - 4 * xpow0 * xpow1) % 8
             if tpow0 > 0:
-                circuit.u1(tpow0 * np.pi / 4, 0)
+                circuit.append(U1Gate(tpow0 * np.pi / 4), [0])
             if xpow0 == 1:
                 circuit.x(0)
             if tpow1 > 0:
-                circuit.u1(tpow1 * np.pi / 4, 1)
+                circuit.append(U1Gate(tpow1 * np.pi / 4), [1])
             if xpow1 == 1:
                 circuit.x(1)
             # CS gate is implemented using 2 CX gates
-            circuit.u1(np.pi / 4, 0)
-            circuit.u1(np.pi / 4, 1)
+            circuit.append(U1Gate(np.pi / 4), [0])
+            circuit.append(U1Gate(np.pi / 4), [1])
             circuit.cx(0, 1)
-            circuit.u1(7 * np.pi / 4, 1)
+            circuit.append(U1Gate(7 * np.pi / 4), [1])
             circuit.cx(0, 1)
 
         # CSdg-like class
@@ -1044,18 +1050,18 @@ def decompose_cnotdihedral_2_qubits(elem):
             tpow0 = (weight_1[0] - 6 * xpow1 - 4 * xpow0 * xpow1) % 8
             tpow1 = (weight_1[1] - 6 * xpow0 - 4 * xpow0 * xpow1) % 8
             if tpow0 > 0:
-                circuit.u1(tpow0 * np.pi / 4, 0)
+                circuit.append(U1Gate(tpow0 * np.pi / 4), [0])
             if xpow0 == 1:
                 circuit.x(0)
             if tpow1 > 0:
-                circuit.u1(tpow1 * np.pi / 4, 1)
+                circuit.append(U1Gate(tpow1 * np.pi / 4), [1])
             if xpow1 == 1:
                 circuit.x(1)
             # CSdg gate is implemented using 2 CX gates
-            circuit.u1(7 * np.pi / 4, 0)
-            circuit.u1(7 * np.pi / 4, 1)
+            circuit.append(U1Gate(7 * np.pi / 4), [0])
+            circuit.append(U1Gate(7 * np.pi / 4), [1])
             circuit.cx(0, 1)
-            circuit.u1(np.pi / 4, 1)
+            circuit.append(U1Gate(np.pi / 4), [1])
             circuit.cx(0, 1)
 
         # CZ-like class
@@ -1063,11 +1069,11 @@ def decompose_cnotdihedral_2_qubits(elem):
             tpow0 = (weight_1[0] - 4 * xpow1) % 8
             tpow1 = (weight_1[1] - 4 * xpow0) % 8
             if tpow0 > 0:
-                circuit.u1(tpow0 * np.pi / 4, 0)
+                circuit.append(U1Gate(tpow0 * np.pi / 4), [0])
             if xpow0 == 1:
                 circuit.x(0)
             if tpow1 > 0:
-                circuit.u1(tpow1 * np.pi / 4, 1)
+                circuit.append(U1Gate(tpow1 * np.pi / 4), [1])
             if xpow1 == 1:
                 circuit.x(1)
             # CZ gate is implemented using 2 CX gates
@@ -1086,16 +1092,16 @@ def decompose_cnotdihedral_2_qubits(elem):
             tpow0 = (weight_1[0] + m) % 8
             tpow1 = (weight_1[1] + m) % 8
         if tpow0 > 0:
-            circuit.u1(tpow0 * np.pi / 4, 0)
+            circuit.append(U1Gate(tpow0 * np.pi / 4), [0])
         if xpow0 == 1:
             circuit.x(0)
         if tpow1 > 0:
-            circuit.u1(tpow1 * np.pi / 4, 1)
+            circuit.append(U1Gate(tpow1 * np.pi / 4), [1])
         if xpow1 == 1:
             circuit.x(1)
         circuit.cx(0, 1)
         if m > 0:
-            circuit.u1(m * np.pi / 4, 1)
+            circuit.append(U1Gate(m * np.pi / 4), [1])
 
     # CX10-like class
     if (linear == [[1, 1], [0, 1]]).all():
@@ -1110,16 +1116,16 @@ def decompose_cnotdihedral_2_qubits(elem):
             tpow0 = (weight_1[0] + m) % 8
             tpow1 = (weight_1[1] + m) % 8
         if tpow0 > 0:
-            circuit.u1(tpow0 * np.pi / 4, 0)
+            circuit.append(U1Gate(tpow0 * np.pi / 4), [0])
         if xpow0 == 1:
             circuit.x(0)
         if tpow1 > 0:
-            circuit.u1(tpow1 * np.pi / 4, 1)
+            circuit.append(U1Gate(tpow1 * np.pi / 4), [1])
         if xpow1 == 1:
             circuit.x(1)
         circuit.cx(1, 0)
         if m > 0:
-            circuit.u1(m * np.pi / 4, 0)
+            circuit.append(U1Gate(m * np.pi / 4), [0])
 
     # CX01*CX10-like class
     if (linear == [[0, 1], [1, 1]]).all():
@@ -1134,17 +1140,17 @@ def decompose_cnotdihedral_2_qubits(elem):
             tpow0 = (weight_1[0] + m) % 8
             tpow1 = (weight_1[1] + m) % 8
         if tpow0 > 0:
-            circuit.u1(tpow0 * np.pi / 4, 0)
+            circuit.append(U1Gate(tpow0 * np.pi / 4), [0])
         if xpow0 == 1:
             circuit.x(0)
         if tpow1 > 0:
-            circuit.u1(tpow1 * np.pi / 4, 1)
+            circuit.append(U1Gate(tpow1 * np.pi / 4), [1])
         if xpow1 == 1:
             circuit.x(1)
         circuit.cx(0, 1)
         circuit.cx(1, 0)
         if m > 0:
-            circuit.u1(m * np.pi / 4, 1)
+            circuit.append(U1Gate(m * np.pi / 4), [1])
 
     # CX10*CX01-like class
     if (linear == [[1, 1], [1, 0]]).all():
@@ -1159,17 +1165,17 @@ def decompose_cnotdihedral_2_qubits(elem):
             tpow0 = (weight_1[0] + m) % 8
             tpow1 = (weight_1[1] + m) % 8
         if tpow0 > 0:
-            circuit.u1(tpow0 * np.pi / 4, 0)
+            circuit.append(U1Gate(tpow0 * np.pi / 4), [0])
         if xpow0 == 1:
             circuit.x(0)
         if tpow1 > 0:
-            circuit.u1(tpow1 * np.pi / 4, 1)
+            circuit.append(U1Gate(tpow1 * np.pi / 4), [1])
         if xpow1 == 1:
             circuit.x(1)
         circuit.cx(1, 0)
         circuit.cx(0, 1)
         if m > 0:
-            circuit.u1(m * np.pi / 4, 0)
+            circuit.append(U1Gate(m * np.pi / 4), [0])
 
     # CX01*CX10*CX01-like class
     if (linear == [[0, 1], [1, 0]]).all():
@@ -1184,17 +1190,17 @@ def decompose_cnotdihedral_2_qubits(elem):
             tpow0 = (weight_1[0] + m) % 8
             tpow1 = (weight_1[1] + m) % 8
         if tpow0 > 0:
-            circuit.u1(tpow0 * np.pi / 4, 0)
+            circuit.append(U1Gate(tpow0 * np.pi / 4), [0])
         if xpow0 == 1:
             circuit.x(0)
         if tpow1 > 0:
-            circuit.u1(tpow1 * np.pi / 4, 1)
+            circuit.append(U1Gate(tpow1 * np.pi / 4), [1])
         if xpow1 == 1:
             circuit.x(1)
         circuit.cx(0, 1)
         circuit.cx(1, 0)
         if m > 0:
-            circuit.u1(m * np.pi / 4, 1)
+            circuit.append(U1Gate(m * np.pi / 4), [1])
         circuit.cx(0, 1)
 
     return circuit
@@ -1274,7 +1280,7 @@ def decompose_cnotdihedral_general(elem):
                     new_elem.cnot(j, k)
                     new_circuit.cx(i, k)
                     new_circuit.cx(j, k)
-                    new_circuit.u1(np.pi / 4, k)
+                    new_circuit.append(U1Gate(np.pi / 4), [k])
                     new_circuit.cx(i, k)
                     new_circuit.cx(j, k)
 
@@ -1289,7 +1295,7 @@ def decompose_cnotdihedral_general(elem):
                 new_elem.phase(tpow, j)
                 new_elem.cnot(i, j)
                 new_circuit.cx(i, j)
-                new_circuit.u1(tpow * np.pi / 4, j)
+                new_circuit.append(U1Gate(tpow * np.pi / 4), [j])
                 new_circuit.cx(i, j)
 
     # Do u1 gates to construct all monomials of weight 1
@@ -1299,7 +1305,7 @@ def decompose_cnotdihedral_general(elem):
         tpow = (tpow1 - tpow2) % 8
         if tpow != 0:
             new_elem.phase(tpow, i)
-            new_circuit.u1(tpow * np.pi / 4, i)
+            new_circuit.append(U1Gate(tpow * np.pi / 4), [i])
 
     if elem.poly != new_elem.poly:
         raise QiskitError("Could not recover phase polynomial.")
